@@ -25,10 +25,10 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_mc146818.c,v 1.72 2005/08/02 07:56:37 debug Exp $
+ *  $Id: dev_mc146818.c,v 1.75 2005/10/09 22:21:31 debug Exp $
  *  
  *  MC146818 real-time clock, used by many different machines types.
- *  (DS1687 as used in some SGI machines is similar to MC146818.)
+ *  (DS1687 as used in some other machines is also similar to the MC146818.)
  *
  *  This device contains Date/time, the machine's ethernet address (on
  *  DECstation 3100), and can cause periodic (hardware) interrupts.
@@ -222,10 +222,16 @@ static void mc146818_update_time(struct mc_data *d)
 	d->reg[4 * MC_MONTH] = tmp->tm_mon + 1;
 	d->reg[4 * MC_YEAR]  = tmp->tm_year;
 
+	/*
+	 *  Special hacks for emulating the behaviour of various machines:
+	 */
 	switch (d->access_style) {
 	case MC146818_ARC_NEC:
 		d->reg[4 * MC_YEAR] += (0x18 - 104);
 		break;
+	case MC146818_CATS:
+		d->reg[4 * MC_YEAR] %= 100;
+		break;
 	case MC146818_SGI:
 		/*
 		 *  NetBSD/sgimips assumes data in BCD format.
@@ -247,10 +253,8 @@ static void mc146818_update_time(struct mc_data *d)
 			  (d->reg[4 * MC_YEAR] - 30 + 40)
 			: (d->reg[4 * MC_YEAR] - 40)
 		      );
-
 		/*  Century:  */
 		d->reg[72 * 4] = 19 + (tmp->tm_year / 100);
-
 		break;
 	case MC146818_DEC:
 		/*
@@ -303,8 +307,9 @@ int dev_mc146818_access(struct cpu *cpu, struct memory *mem,
 
 	/*  Different ways of accessing the registers:  */
 	switch (d->access_style) {
+	case MC146818_CATS:
 	case MC146818_PC_CMOS:
-		if (relative_addr == 0x70 || relative_addr == 0x00) {
+		if ((relative_addr & 1) == 0x00) {
 			if (writeflag == MEM_WRITE) {
 				d->last_addr = data[0];
 				return 1;
@@ -312,12 +317,8 @@ int dev_mc146818_access(struct cpu *cpu, struct memory *mem,
 				data[0] = d->last_addr;
 				return 1;
 			}
-		} else if (relative_addr == 0x71 || relative_addr == 0x01)
+		} else
 			relative_addr = d->last_addr * 4;
-		else {
-			fatal("[ mc146818: not accessed as an "
-			    "MC146818_PC_CMOS device! ]\n");
-		}
 		break;
 	case MC146818_ARC_NEC:
 		if (relative_addr == 0x01) {
@@ -630,6 +631,7 @@ void dev_mc146818_init(struct machine *machine, struct memory *mem,
 
 	dev_len = DEV_MC146818_LENGTH;
 	switch (access_style) {
+	case MC146818_CATS:
 	case MC146818_PC_CMOS:
 		dev_len = 2;
 		break;
