@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_mips.h,v 1.20 2005/08/07 17:42:03 debug Exp $
+ *  $Id: cpu_mips.h,v 1.21 2005/08/28 20:16:24 debug Exp $
  */
 
 #include "misc.h"
@@ -405,7 +405,7 @@ void mips_coproc_tlb_set_entry(struct cpu *cpu, int entrynr, int size,
 void mips_update_translation_table(struct cpu *cpu, uint64_t vaddr_page,
         unsigned char *host_page, int writeflag, uint64_t paddr_page);
 void clear_all_chunks_from_all_tables(struct cpu *cpu);
-void mips_invalidate_translation_caches_paddr(struct cpu *cpu, uint64_t paddr);
+void mips_invalidate_translation_caches_paddr(struct cpu *cpu, uint64_t, int);
 void coproc_register_read(struct cpu *cpu,
         struct mips_coproc *cp, int reg_nr, uint64_t *ptr, int select);
 void coproc_register_write(struct cpu *cpu,
