@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_ppc.h,v 1.29 2005/08/14 23:44:23 debug Exp $
+ *  $Id: cpu_ppc.h,v 1.38 2005/09/24 23:44:19 debug Exp $
  */
 
 #include "misc.h"
@@ -42,13 +42,17 @@ struct cpu_family;
 /*  PPC CPU types:  */
 struct ppc_cpu_type_def { 
 	char		*name;
+	int		pvr;
 	int		bits;
 	int		flags;
 	int		icache_shift;
+	int		ilinesize;
 	int		iway;
 	int		dcache_shift;
+	int		dlinesize;
 	int		dway;
 	int		l2cache_shift;
+	int		l2linesize;
 	int		l2way;
 	int		altivec;
 
@@ -59,15 +63,15 @@ struct ppc_cpu_type_def {
 #define	PPC_NOFP		1
 /*  TODO: Most of these just bogus  */
 
-#define PPC_CPU_TYPE_DEFS	{				\
-	{ "PPC405GP", 32, PPC_NOFP, 15, 2, 15, 2, 20, 1, 0 },	\
-	{ "PPC603e", 32, 0, 14, 4, 14, 4, 0, 0, 0 },		\
-	{ "MPC7400", 32, 0, 15, 2, 15, 2, 19, 1, 1 },		\
-	{ "PPC750", 32, 0, 15, 2, 15, 2, 20, 1, 0 },		\
-	{ "G4e", 32, 0, 15, 8, 15, 8, 18, 8, 1 },		\
-	{ "PPC970", 64, 0, 16, 1, 15, 2, 19, 1, 1 },		\
-	{ NULL, 0, 0, 0,0, 0,0, 0,0, 0 }			\
-	};
+#define PPC_CPU_TYPE_DEFS	{					\
+	{ "PPC405GP",	0,          32, PPC_NOFP, 15,5,2, 15,5,2, 20,5,1, 0 }, \
+	{ "PPC603e",	0,          32, 0, 14,5,4, 14,5,4, 0,0,0, 0 },	\
+	{ "MPC7400",	0x000c0000, 32, 0, 15,5,2, 15,5,2, 19,5,1, 1 },	\
+	{ "PPC750",	0x00084202, 32, 0, 15,5,2, 15,5,2, 20,5,1, 0 },	\
+	{ "G4e",	0,          32, 0, 15,5,8, 15,5,8, 18,5,8, 1 },	\
+	{ "PPC970",	0x00390000, 64, 0, 16,7,1, 15,7,2, 19,7,1, 1 },	\
+	{ NULL,		0,          0,0,0,0,0,0,0,0,0,0,0,0 }		\
+	}
 
 #define	PPC_NGPRS		32
 #define	PPC_NFPRS		32
@@ -111,7 +115,6 @@ struct ppc_cpu {
 	struct ppc_cpu_type_def cpu_type;
 
 	uint64_t	of_emul_addr;
-	uint64_t	pc_last;
 
 	int		mode;		/*  MODE_PPC or MODE_POWER  */
 	int		bits;		/*  32 or 64  */
@@ -130,6 +133,9 @@ struct ppc_cpu {
 	uint32_t	tbu;		/*  Time Base Upper  */
 	uint32_t	dec;		/*  Decrementer  */
 	uint32_t	hdec;		/*  Hypervisor Decrementer  */
+	uint64_t	sdr1;		/*  Storage Descriptor Register  */
+	uint64_t	srr0;		/*  Supervisor save/restore 0  */
+	uint64_t	srr1;		/*  Supervisor save/restore 1  */
 	uint64_t	ssr0;		/*  Machine status save/restore
 					    register 0  */
 	uint64_t	ssr1;		/*  Machine status save/restore
@@ -139,9 +145,22 @@ struct ppc_cpu {
 	uint64_t	sprg1;		/*  Special Purpose Register G1  */
 	uint64_t	sprg2;		/*  Special Purpose Register G2  */
 	uint64_t	sprg3;		/*  Special Purpose Register G3  */
+	uint64_t	dbsr;		/*  Debug Status Register  */
 	uint32_t	pvr;		/*  Processor Version Register  */
 	uint32_t	pir;		/*  Processor ID  */
 
+	/*  TODO: 64-bit SRs? (Segment registers)  */
+	uint32_t	sr[16];
+
+	/*  TODO: 64-bit BATs?  */
+	uint32_t	ibat_u[4];
+	uint32_t	ibat_l[4];
+	uint32_t	dbat_u[4];
+	uint32_t	dbat_l[4];
+
+	uint64_t	ll_addr;	/*  Load-linked / store-conditional  */
+	int		ll_bit;
+
 
 	/*
 	 *  Instruction translation cache:
@@ -200,11 +219,18 @@ struct ppc_cpu {
 /*  cpu_ppc.c:  */
 void ppc_update_translation_table(struct cpu *cpu, uint64_t vaddr_page,
 	unsigned char *host_page, int writeflag, uint64_t paddr_page);
-void ppc_invalidate_translation_caches_paddr(struct cpu *cpu, uint64_t paddr);
-void ppc_invalidate_code_translation_caches(struct cpu *cpu);
+void ppc32_update_translation_table(struct cpu *cpu, uint64_t vaddr_page,
+	unsigned char *host_page, int writeflag, uint64_t paddr_page);
+void ppc_invalidate_translation_caches_paddr(struct cpu *cpu, uint64_t, int);
+void ppc32_invalidate_translation_caches_paddr(struct cpu *cpu, uint64_t, int);
+void ppc_invalidate_code_translation(struct cpu *cpu, uint64_t, int);
+void ppc32_invalidate_code_translation(struct cpu *cpu, uint64_t, int);
 int ppc_memory_rw(struct cpu *cpu, struct memory *mem, uint64_t vaddr,
 	unsigned char *data, size_t len, int writeflag, int cache_flags);
 int ppc_cpu_family_init(struct cpu_family *);
 
+/*  memory_ppc.c:  */
+int ppc_translate_address(struct cpu *cpu, uint64_t vaddr,
+	uint64_t *return_addr, int flags);
 
 #endif	/*  CPU_PPC_H  */
